@@ -415,7 +415,11 @@ display_handle_delete_id(void *data, struct wl_display *display, uint32_t id)
 	pthread_mutex_lock(&display->mutex);
 
 	proxy = wl_map_lookup(&display->objects, id);
-	if (proxy != WL_ZOMBIE_OBJECT)
+
+	if (!proxy)
+		wl_log("error: received delete_id for unknown id (%u)\n", id);
+
+	if (proxy && proxy != WL_ZOMBIE_OBJECT)
 		proxy->flags |= WL_PROXY_FLAG_ID_DELETED;
 	else
 		wl_map_remove(&display->objects, id);
@@ -669,21 +673,21 @@ create_proxies(struct wl_proxy *sender, struct wl_closure *closure)
 	int count;
 
 	signature = closure->message->signature;
-	count = arg_count_for_signature(signature) + 2;
-	for (i = 2; i < count; i++) {
+	count = arg_count_for_signature(signature);
+	for (i = 0; i < count; i++) {
 		signature = get_next_argument(signature, &arg);
 		switch (arg.type) {
 		case 'n':
-			id = **(uint32_t **) closure->args[i];
+			id = closure->args[i].n;
 			if (id == 0) {
-				*(void **) closure->args[i] = NULL;
+				closure->args[i].o = NULL;
 				break;
 			}
 			proxy = wl_proxy_create_for_id(sender, id,
-						       closure->message->types[i - 2]);
+						       closure->message->types[i]);
 			if (proxy == NULL)
 				return -1;
-			*(void **) closure->args[i] = proxy;
+			closure->args[i].o = (struct wl_object *)proxy;
 			break;
 		default:
 			break;
@@ -702,13 +706,13 @@ increase_closure_args_refcount(struct wl_closure *closure)
 	struct wl_proxy *proxy;
 
 	signature = closure->message->signature;
-	count = arg_count_for_signature(signature) + 2;
-	for (i = 2; i < count; i++) {
+	count = arg_count_for_signature(signature);
+	for (i = 0; i < count; i++) {
 		signature = get_next_argument(signature, &arg);
 		switch (arg.type) {
 		case 'n':
 		case 'o':
-			proxy = *(struct wl_proxy **) closure->args[i];
+			proxy = (struct wl_proxy *) closure->args[i].o;
 			if (proxy)
 				proxy->refcount++;
 			break;
@@ -779,16 +783,16 @@ decrease_closure_args_refcount(struct wl_closure *closure)
 	struct wl_proxy *proxy;
 
 	signature = closure->message->signature;
-	count = arg_count_for_signature(signature) + 2;
-	for (i = 2; i < count; i++) {
+	count = arg_count_for_signature(signature);
+	for (i = 0; i < count; i++) {
 		signature = get_next_argument(signature, &arg);
 		switch (arg.type) {
 		case 'n':
 		case 'o':
-			proxy = *(struct wl_proxy **) closure->args[i];
+			proxy = (struct wl_proxy *) closure->args[i].o;
 			if (proxy) {
 				if (proxy->flags & WL_PROXY_FLAG_DESTROYED)
-					*(void **) closure->args[i] = NULL;
+					closure->args[i].o = NULL;
 
 				proxy->refcount--;
 				if (!proxy->refcount)
@@ -812,7 +816,7 @@ dispatch_event(struct wl_display *display, struct wl_event_queue *queue)
 	closure = container_of(queue->event_list.next,
 			       struct wl_closure, link);
 	wl_list_remove(&closure->link);
-	opcode = closure->buffer[1] & 0xffff;
+	opcode = closure->opcode;
 
 	/* Verify that the receiving object is still valid by checking if has
 	 * been destroyed by the application. */
@@ -822,10 +826,10 @@ dispatch_event(struct wl_display *display, struct wl_event_queue *queue)
 	proxy_destroyed = !!(proxy->flags & WL_PROXY_FLAG_DESTROYED);
 
 	proxy->refcount--;
-	if (!proxy->refcount)
-		free(proxy);
-
 	if (proxy_destroyed) {
+		if (!proxy->refcount)
+			free(proxy);
+
 		wl_closure_destroy(closure);
 		return;
 	}
@@ -836,8 +840,8 @@ dispatch_event(struct wl_display *display, struct wl_event_queue *queue)
 		if (wl_debug)
 			wl_closure_print(closure, &proxy->object, false);
 
-		wl_closure_invoke(closure, &proxy->object,
-				  proxy->object.implementation[opcode],
+		wl_closure_invoke(closure, WL_CLOSURE_INVOKE_CLIENT,
+				  &proxy->object, opcode,
 				  proxy->user_data);
 	}
 
@@ -1052,7 +1056,7 @@ wl_display_get_error(struct wl_display *display)
 	return ret;
 }
 
-/** Send all buffered request on the display to the server
+/** Send all buffered requests on the display to the server
  *
  * \param display The display context object
  * \return The number of bytes send on success or -1 on failure
@@ -1062,6 +1066,11 @@ wl_display_get_error(struct wl_display *display)
  * of bytes sent to the server is returned. On failure, this
  * function returns -1 and errno is set appropriately.
  *
+ * wl_display_flush() never blocks.  It will write as much data as
+ * possible, but if all data could not be written, errno will be set
+ * to EAGAIN and -1 returned.  In that case, use poll on the display
+ * file descriptor to wait for it to become writable again.
+ *
  * \memberof wl_display
  */
 WL_EXPORT int
@@ -1127,6 +1136,18 @@ wl_proxy_get_id(struct wl_proxy *proxy)
 	return proxy->object.id;
 }
 
+/** Get the interface name (class) of a proxy object
+ *
+ * \param proxy The proxy object
+ * \return The interface name of the object associated with the proxy
+ *
+ * \memberof wl_proxy
+ */
+WL_EXPORT const char *
+wl_proxy_get_class(struct wl_proxy *proxy)
+{
+	return proxy->object.interface->name;
+}
 
 /** Assign a proxy to an event queue
  *
