@@ -191,23 +191,6 @@ wl_resource_post_error(struct wl_resource *resource,
 			       WL_DISPLAY_ERROR, resource, code, buffer);
 }
 
-static void
-deref_new_objects(struct wl_closure *closure)
-{
-	const char *signature;
-	int i;
-
-	signature = closure->message->signature;
-	for (i = 0; signature[i]; i++) {
-		switch (signature[i]) {
-		case 'n':
-			closure->args[i + 2] = *(uint32_t **) closure->args[i + 2];
-			closure->types[i] = &ffi_type_uint32;
-			break;
-		}
-	}
-}
-
 static int
 wl_client_connection_data(int fd, uint32_t mask, void *data)
 {
@@ -277,8 +260,11 @@ wl_client_connection_data(int fd, uint32_t mask, void *data)
 						  &client->objects, message);
 		len -= size;
 
-		if ((closure == NULL && errno == EINVAL) ||
-		    wl_closure_lookup_objects(closure, &client->objects) < 0) {
+		if (closure == NULL && errno == ENOMEM) {
+			wl_resource_post_no_memory(resource);
+			break;
+		} else if ((closure == NULL && errno == EINVAL) ||
+			   wl_closure_lookup_objects(closure, &client->objects) < 0) {
 			wl_resource_post_error(client->display_resource,
 					       WL_DISPLAY_ERROR_INVALID_METHOD,
 					       "invalid arguments for %s@%u.%s",
@@ -286,18 +272,13 @@ wl_client_connection_data(int fd, uint32_t mask, void *data)
 					       object->id,
 					       message->name);
 			break;
-		} else if (closure == NULL && errno == ENOMEM) {
-			wl_resource_post_no_memory(resource);
-			break;
 		}
 
 		if (wl_debug)
 			wl_closure_print(closure, object, false);
 
-		deref_new_objects(closure);
-
-		wl_closure_invoke(closure, object,
-				  object->implementation[opcode], client);
+		wl_closure_invoke(closure, WL_CLOSURE_INVOKE_SERVER, object,
+				  opcode, client);
 
 		wl_closure_destroy(closure);
 
@@ -965,6 +946,31 @@ wl_pointer_end_grab(struct wl_pointer *pointer)
 			 pointer->current_x, pointer->current_y);
 }
 
+static void
+current_surface_destroy(struct wl_listener *listener, void *data)
+{
+	struct wl_pointer *pointer =
+		container_of(listener, struct wl_pointer, current_listener);
+
+	pointer->current = NULL;
+}
+
+WL_EXPORT void
+wl_pointer_set_current(struct wl_pointer *pointer, struct wl_surface *surface)
+{
+	if (pointer->current)
+		wl_list_remove(&pointer->current_listener.link);
+
+	pointer->current = surface;
+
+	if (!surface)
+		return;
+	
+	wl_signal_add(&surface->resource.destroy_signal,
+		      &pointer->current_listener);
+	pointer->current_listener.notify = current_surface_destroy;
+}
+
 WL_EXPORT void
 wl_touch_start_grab(struct wl_touch *touch, struct wl_touch_grab *grab)
 {
@@ -1415,13 +1421,9 @@ wl_client_add_object(struct wl_client *client,
 		return NULL;
 	}
 
-	resource->object.interface = interface;
-	resource->object.implementation = implementation;
-	resource->object.id = id;
+	wl_resource_init(resource, interface, implementation, id, data);
 	resource->client = client;
-	resource->data = data;
 	resource->destroy = (void *) free;
-	wl_signal_init(&resource->destroy_signal);
 
 	if (wl_map_insert_at(&client->objects, resource->object.id, resource) < 0) {
 		wl_resource_post_error(client->display_resource,
